/*   ArgoCraft Video Capture Board    */
/*   Driver and Sample Applications   */
/* Version 1.32 for Linux and FreeBSD */
/* Copyright (C) 1996 ArgoCraft Corp. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/time.h>

/*
   acvc.h $B$K$O%G%P%$%9$rMxMQ$9$k$N$KJXMx$JDj?t$H%^%/%m$,Dj5A$5$l$F$$$^$9!#(B
   $BDj?t$d%^%/%m$NL>A0$O(B ACVC_ $B$G;O$^$j$^$9!#(B
*/

#include "acvc.h"

/*
   $B%-%c%W%A%c$7$?2hA|$rJ]B8$9$k$N$K==J,$JBg$-$5$N%P%C%U%!$rMQ0U$7$^$9!#(B
   $B$b$A$m$sI,MW$K1~$8$F(B malloc $B$7$F$b9=$$$^$;$s!#(B
*/

static char buf [ACVC_BUFSIZE];

/*
   usage $B$rI=<($7$FH4$1$k%5%V%k!<%A%s$G$9!#(B
*/

void usage (void) {
  fprintf (stderr, "usage: vctest [-h] size_x size_y color ");
  fprintf (stderr, "view_x view_y mono src bri con sat\n");
  exit (1);
}

int main (int argc, char ** argv) {
  unsigned int size;
  unsigned short x, y;
  int h, c, fd, ret, i;
  struct timeval tv1, tv2;
  struct timezone tz;
  unsigned short * p;
  unsigned long * q;

  /*
     $B0z?t$N?t$,9g$o$J$$$H$-!"(Busage $B$rI=<($7$FH4$1$^$9!#(B
  */

  if (argc <= 1) {
    usage ();
  }

  /*
     -h $B%*%W%7%g%s$,;XDj$5$l$F$$$?$i(B $BJQ?t(B h $B$r(B 1 $B$K!";XDj$5$l$F$$$J$+$C$?$i(B
     0 $B$K$7$^$9!#(B
  */

  if (! strcmp (argv [1], "-h")) {
    h = 1;
    argc --;
    argv ++;
  } else {
    h = 0;
  }

  /*
     $B0z?t$N?t$,9g$o$J$$$H$-!"(Busage $B$rI=<($7$FH4$1$^$9!#(B
  */

  if (argc != 11) {
    usage ();
  }

  /*
     $B=PNO2hA|$NBg$-$5$r(B $BJQ?t(B x $B$H(B y $B$KF~$l$^$9!#$^$?!"%-%c%W%A%c$9$k2hA|$r(B
     1 $B%T%/%;%kEv$?$j(B 32 bit $B$GI=8=$9$k$H$-(B c $B$r(B 1 $B$K!"(B16 bit $B$GI=8=$9$k$H$-(B
     0 $B$K(B $B$7$^$9!#(Bsize $B$O(B $B%-%c%W%A%c$7$?2hA|$r#1%U%l!<%`J]B8$9$k$?$a$KI,MW$J(B
     $B%P%$%H?t$G$9!#(B
  */

  x = (unsigned short) atoi (argv [1]);
  y = (unsigned short) atoi (argv [2]);

  c = atoi (argv [3]) ? 1 : 0;

  size = x * y * (c ? 4 : 2);

  /*
     $B%G%P%$%9%U%!%$%k$r%*!<%W%s$7$^$9!#(B/dev/acvc0 $B$O6v?t%U%#!<%k%I!"(B
     /dev/acvc $B$ON>%U%#!<%k%I$N9g@.$rI=$9%G%P%$%9%U%!%$%k$G$9!#(B
     /dev/acvc1 $B$O>-Mh$N3HD%$N$?$aM=Ls$5$l$F$$$^$9!#(B
  */

  if (h) {
    if ((fd = open ("/dev/acvc0", O_RDONLY)) < 0) {
      perror ("vctest: open");
      exit (1);
    }
  } else {
    if ((fd = open ("/dev/acvc", O_RDONLY)) < 0) {
      perror ("vctest: open");
      exit (1);
    }
  }

  /*
     ACVC_SETSIZE $B$G%-%c%W%A%c$9$k2hA|$N2rA|EY$H?'?t$r@_Dj$7$^$9!#Bh#10z?t$O(B
     $B%*!<%W%s$5$l$?%G%P%$%9%U%!%$%k$rI=$9%U%!%$%k%G%#%9%/%j%W%?!"Bh#20z?t$O(B
     x $BJ}8~$N2rA|EY!"Bh#30z?t$O(B y $BJ}8~$N2rA|EY!"Bh#40z?t$O#1%T%/%;%kEv$?$j$N(B
     bit $B?t$r(B 0 $B$^$?$O(B 1 $B$GI=$7$?$b$N$G!"(B0 $B$,(B 16 bit $B!"(B1 $B$,(B 32 bit $B$rI=$7$^$9!#(B
     $B0z?t$NHO0O$K4X$9$k@)8B$K$D$$$F$O!"%\!<%I$N@bL@=q$r;2>H$7$F$/$@$5$$!#(B
     $BFC$K@_Dj$7$J$+$C$?>l9g$N%G%U%)%k%HCM$O!"(B/dev/acvc $B$G(B 640 x 480 x 32 bpp$B!"(B
     /dev/acvc0 $B$*$h$S(B /dev/acvc1 $B$G(B 320 x 240 x 32 bpp $B$G$9!#(B

     $BLa$jCM$O@5>o$K@_Dj$5$l$?>l9g(B 0 $B$K!"%(%i!<$,5/$3$C$?>l9g(B -1 $B$K$J$j$^$9!#(B
     $B%(%i!<$,5/$3$C$?>l9g!"860x$K1~$8$F(B errno $B$,%;%C%H$5$l$^$9!#(B
  */

  if (ACVC_SETSIZE (fd, x, y, c)) {
    perror ("vctest: SETSIZE");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     ACVC_SETVIEW $B$GF~NO1GA|$N$&$A%-%c%W%A%c$9$kHO0O$r;XDj$7$^$9!#Bh#10z?t$O(B
     $B%*!<%W%s$5$l$?%G%P%$%9%U%!%$%k$rI=$9%U%!%$%k%G%#%9%/%j%W%?!"Bh#20z?t$O(B
     x $BJ}8~$NBg$-$5!"Bh#30z?t$O(B y $BJ}8~$NBg$-$5$G$9!#(Bx = 640, y = 480 $B$H$9$k$H(B
     $B2hLLA4BN$,%-%c%W%A%c$5$l$^$9$,!"%F%l%S$J$I$N=PNO$r%-%c%W%A%c$9$k>l9g$O(B
     $BC<$NJ}$,6uGr$K$J$C$F$$$k$3$H$,B?$$$N$G!">/$7>.$5$a$K$7$?J}$,NI$$$G$7$g$&!#(B
     $BFC$K@_Dj$7$J$+$C$?>l9g$N%G%U%)%k%HCM$O(B 640 x 480 $B$G$9$,!"$3$NCM$O>-Mh(B
     $BJQ99$5$l$k2DG=@-$,$"$k$N$G!"0MB8$7$J$$$G$/$@$5$$!#CM$N@)8B$K$D$$$F$O!"(B
     $B%\!<%I$N@bL@=q$r;2>H$7$F$/$@$5$$!#(B

     $BLa$jCM$O@5>o$K@_Dj$5$l$?>l9g(B 0 $B$K!"%(%i!<$,5/$3$C$?>l9g(B -1 $B$K$J$j$^$9!#(B
     $B%(%i!<$,5/$3$C$?>l9g!"860x$K1~$8$F(B errno $B$,%;%C%H$5$l$^$9!#(B
  */

  if (ACVC_SETVIEW (fd, atoi (argv [4]), atoi (argv [5]))) {
    perror ("vctest: SETVIEW");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     ACVC_SETMONO $B$G%-%c%W%A%c$r%+%i!<$G9T$J$&$+!"%b%N%/%m$G9T$J$&$+$r(B
     $B;XDj$7$^$9!#Bh#10z?t$O%*!<%W%s$5$l$?%G%P%$%9%U%!%$%k$rI=$9%U%!%$%k(B
     $B%G%#%9%/%j%W%?!"Bh#20z?t$O%+%i!<$N$H$-(B 0 $B$r!"%b%N%/%m$N$H$-(B 1 $B$r(B
     $BEO$7$^$9!#%b%N%/%m$r;XDj$9$k$H$-$N@)8B;v9`$K$D$$$F$O!"%\!<%I$N@bL@=q$r(B
     $B;2>H$7$F$/$@$5$$!#FC$K@_Dj$7$J$+$C$?>l9g$N%G%U%)%k%HCM$O(B 0 $B$G$9!#(B

     $BLa$jCM$O@5>o$K@_Dj$5$l$?>l9g(B 0 $B$K!"%(%i!<$,5/$3$C$?>l9g(B -1 $B$K$J$j$^$9!#(B
     $B%(%i!<$,5/$3$C$?>l9g!"860x$K1~$8$F(B errno $B$,%;%C%H$5$l$^$9!#(B
  */

  if (ACVC_SETMONO (fd, atoi (argv [6]))) {
    perror ("vctest: SETMONO");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     ACVC_SETSRC $B$GF~NO7OE}$rA*Br$7$^$9!#Bh#10z?t$O%*!<%W%s$5$l$?%G%P%$%9(B
     $B%U%!%$%k$rI=$9%U%!%$%k%G%#%9%/%j%W%?!"Bh#20z?t$OF~NO7OE}$NHV9f$G$9!#(B
     $BF~NO$O#67OE}$"$k$N$G!"Bh#20z?t$NHO0O$O(B 0 $B0J>e(B 5 $B0J2<$G$9!#FC$K@_Dj(B
     $B$7$J$+$C$?>l9g$N%G%U%)%k%HCM$O(B 0 $B$G$9!#(B

     $BLa$jCM$O@5>o$K@_Dj$5$l$?>l9g(B 0 $B$K!"%(%i!<$,5/$3$C$?>l9g(B -1 $B$K$J$j$^$9!#(B
     $B%(%i!<$,5/$3$C$?>l9g!"860x$K1~$8$F(B errno $B$,%;%C%H$5$l$^$9!#(B
  */

  if (ACVC_SETSRC (fd, atoi (argv [7]))) {
    perror ("vctest: SETSRC");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     ACVC_SETBRI $B$GL@$k$5$r@_Dj$7$^$9!#Bh#10z?t$O%*!<%W%s$5$l$?%G%P%$%9(B
     $B%U%!%$%k$rI=$9%U%!%$%k%G%#%9%/%j%W%?!"Bh#20z?t$OL@$k$5$rI=$9(B 0 $B0J>e(B
     255 $B0J2<$N@0?t$G$9!#?tCM$,Bg$-$$$[$IL@$k$/$J$j$^$9!#FC$K@_Dj$7$J$+$C$?(B
     $B>l9g$N%G%U%)%k%HCM$O(B 139 $B$G$9$,!"$3$NCM$O>-MhJQ99$5$l$k2DG=@-$,$"$k$N$G!"(B
     $B0MB8$7$J$$$G$/$@$5$$!#(B

     $BLa$jCM$O@5>o$K@_Dj$5$l$?>l9g(B 0 $B$K!"%(%i!<$,5/$3$C$?>l9g(B -1 $B$K$J$j$^$9!#(B
     $B%(%i!<$,5/$3$C$?>l9g!"860x$K1~$8$F(B errno $B$,%;%C%H$5$l$^$9!#(B
  */

  if (ACVC_SETBRI (fd, atoi (argv [8]))) {
    perror ("vctest: SETBRI");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     ACVC_SETCON $B$G%3%s%H%i%9%H$r@_Dj$7$^$9!#Bh#10z?t$O%*!<%W%s$5$l$?%G%P%$%9(B
     $B%U%!%$%k$rI=$9%U%!%$%k%G%#%9%/%j%W%?!"Bh#20z?t$O%3%s%H%i%9%H$rI=$9?tCM(B
     $B!J(B-128 $B0J>e(B 127 $B0J2<$N@0?t!K$G$9!#?tCM$,Bg$-$$$[$I!"$h$j%3%s%H%i%9%H$,(B
     $B6/$/$J$j$^$9!#Ii$N?t$O!"Ii$N%3%s%H%i%9%H$rI=$7$^$9!#FC$K@_Dj$7$J$+$C$?(B
     $B>l9g$N%G%U%)%k%HCM$O(B 70 $B$G$9$,!"$3$NCM$O>-MhJQ99$5$l$k2DG=@-$,$"$k$N$G!"(B
     $B0MB8$7$J$$$G$/$@$5$$!#(B

     $BLa$jCM$O@5>o$K@_Dj$5$l$?>l9g(B 0 $B$K!"%(%i!<$,5/$3$C$?>l9g(B -1 $B$K$J$j$^$9!#(B
     $B%(%i!<$,5/$3$C$?>l9g!"860x$K1~$8$F(B errno $B$,%;%C%H$5$l$^$9!#(B
  */

  if (ACVC_SETCON (fd, atoi (argv [9]))) {
    perror ("vctest: SETCON");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     ACVC_SETSAT $B$G:LEY$r@_Dj$7$^$9!#Bh#10z?t$O%*!<%W%s$5$l$?%G%P%$%9(B
     $B%U%!%$%k$rI=$9%U%!%$%k%G%#%9%/%j%W%?!"Bh#20z?t$O:LEY$rI=$9?tCM(B
     $B!J(B-128 $B0J>e(B 127 $B0J2<$N@0?t!K$G$9!#?t$,Bg$-$$$[$I?'$,6/$/$J$j$^$9!#(B
     $BIi$NCM$O!"Ii$N:LEY$r$"$i$o$7$^$9!#(B0 $B$K$9$k$H?':L$,$J$/$J$j$^$9!#(B
     $BFC$K@_Dj$7$J$+$C$?>l9g$N%G%U%)%k%HCM$O(B 64 $B$G$9$,!"$3$NCM$O>-Mh(B
     $BJQ99$5$l$k2DG=@-$,$"$k$N$G!"0MB8$7$J$$$G$/$@$5$$!#(B

     $BLa$jCM$O@5>o$K@_Dj$5$l$?>l9g(B 0 $B$K!"%(%i!<$,5/$3$C$?>l9g(B -1 $B$K$J$j$^$9!#(B
     $B%(%i!<$,5/$3$C$?>l9g!"860x$K1~$8$F(B errno $B$,%;%C%H$5$l$^$9!#(B
  */

  if (ACVC_SETSAT (fd, atoi (argv [10]))) {
    perror ("vctest: SETSAT");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     $B:G=i$N?t%U%l!<%`$OF14|$,$H$l$F$$$J$$$3$H$,$"$k$N$G!"$"$i$+$8$a(B
     $B%-%c%W%A%c$7$F<N$F$F$*$-$^$9!#(BACVC_CAPSTART $B$H(B ACVC_CAPWAIT $B$K(B
     $B$D$$$F$O8e$G@bL@$5$l$F$$$^$9!#(B
  */

  for (i = 32; i > 0; i --) {
    if (ACVC_CAPSTART (fd)) {
      perror ("vctest: CAPSTART");
      if (close (fd)) {
	perror ("vctest: close");
	exit (1);
      }
      exit (1);
    }
    
    if (ACVC_CAPWAIT (fd)) {
      perror ("vctest: CAPWAIT");
      if (close (fd)) {
	perror ("vctest: close");
	exit (1);
      }
      exit (1);
    }
  }

  /*
     $B%-%c%W%A%c;~4V$rB,Dj$9$k$?$a$K!"8=:_;~9o$r<hF@$7$^$9!#(B
  */

  if (gettimeofday (& tv1, & tz)) {
    perror ("vctest: gettimeofday");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     ACVC_CAPSTART $B$G%-%c%W%A%c$r3+;O$7$^$9!#Bh#10z?t$O%*!<%W%s$5$l$?%G%P%$%9(B
     $B%U%!%$%k$rI=$9%U%!%$%k%G%#%9%/%j%W%?$G$9!#$J$*!"%-%c%W%A%cCf$K%5%$%:$J$I$N(B
     $B%Q%i%a!<%?$rJQ99$9$k$3$H$O$G$-$^$;$s!#JQ99$7$h$&$H$7$?>l9g!"%-%c%W%A%c$,(B
     $B=*N;$9$k$^$G%W%m%;%9$OBT5!$5$;$i$l$^$9!#(B

     $BLa$jCM$O@5>o$K3+;O$5$l$?>l9g(B 0 $B$K!"%(%i!<$,5/$3$C$?>l9g(B -1 $B$K$J$j$^$9!#(B
     $B%(%i!<$,5/$3$C$?>l9g!"860x$K1~$8$F(B errno $B$,%;%C%H$5$l$^$9!#(B

     $B$J$*!"8=:_%-%c%W%A%cCf$+$I$&$+$rCN$j$?$$$H$-$O(B

       ACVC_ISCAPTURING (int fd, int * iscap)

     $B$H$$$&%^%/%m$,MQ0U$5$l$F$$$^$9!#%-%c%W%A%cCf$N>l9g$O(B * iscap $B$K(B 1 $B$,!"(B
     $B%-%c%W%A%cCf$G$J$$>l9g$O(B 0 $B$,%;%C%H$5$l$^$9!#La$jCM$O>e$HF1MM$G$9!#(B

  */

  fprintf (stderr, "vctest: capture start\n");

  if (ACVC_CAPSTART (fd)) {
    perror ("vctest: CAPSTART");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     ACVC_CAPWAIT $B$G%-%c%W%A%c$N=*N;$rBT$A$^$9!#Bh#10z?t$O%*!<%W%s$5$l$?(B
     $B%G%P%$%9%U%!%$%k$rI=$9%U%!%$%k%G%#%9%/%j%W%?$G$9!#(B

     $BLa$jCM$O@5>o$K=*N;$7$?>l9g(B 0 $B$K!"%(%i!<$,5/$3$C$?>l9g(B -1 $B$K$J$j$^$9!#(B
     $B%(%i!<$,5/$3$C$?>l9g!"860x$K1~$8$F(B errno $B$,%;%C%H$5$l$^$9!#(B
  */

  if (ACVC_CAPWAIT (fd)) {
    perror ("vctest: CAPWAIT");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     $B%-%c%W%A%c;~4V$rB,Dj$9$k$?$a$K!"8=:_;~9o$r<hF@$7$^$9!#(B
  */

  if (gettimeofday (& tv2, & tz)) {
    perror ("vctest: gettimeofday");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  fprintf (stderr, "vctest: capture end\n");

  /*
     $B%-%c%W%A%c$K$+$+$C$?;~4V$rI=<($7$^$9!#(B
  */

  fprintf (stderr, "vctest: %ld microseconds\n",
	   (tv2 . tv_usec - tv1 . tv_usec) +
	   (tv2 . tv_sec - tv1 . tv_sec) * 1000000L);

  /*
     $B%-%c%W%A%c$5$l$?2hA|$r%U%l!<%`%P%C%U%!$+$iFI$_9~$_$^$9!#(Bread $B$N(B
     $BBh#30z?t$K$O%U%l!<%`A4BN$rFI$_9~$`$N$K==J,$JCM$r;XDj$7$F$/$@$5$$!#(B
     $B%U%l!<%`%P%C%U%!$rItJ,E*$K(B read $B$9$k$3$H$O$G$-$^$;$s!#%-%c%W%A%cCf$O(B
     $B%U%l!<%`%P%C%U%!$r(B read $B$9$k$3$H$O$G$-$^$;$s!#(Bread $B$7$h$&$H$9$k$H!"(B
     $B%-%c%W%A%c$,=*N;$9$k$^$G%W%m%;%9$OBT5!$5$;$i$l$^$9!#(B
  */

  if ((ret = read (fd, buf, size)) < 0) {
    perror ("vctest: read");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  if (ret != size) {
    fprintf (stderr, "vctest: read failed\n");
    if (close (fd)) {
      perror ("vctest: close");
      exit (1);
    }
    exit (1);
  }

  /*
     $B%G%P%$%9%U%!%$%k$r%/%m!<%:$7$^$9!#(B
  */

  if (close (fd) < 0) {
    perror ("vctest: close");
    exit (1);
  }

  /*
     $B%-%c%W%A%c$5$l$?2hA|$r(B Raw PPM $B7A<0$GI8=`=PNO$K=PNO$7$^$9!#(B

     $B%U%l!<%`%P%C%U%!>e$N2hA|$O!":8>e$+$i1&2<$X1Q8l$NJ8>O$rFI$`=g=x$G(B
     $B=q$-9~$^$l$F$$$^$9!#$"$k#1$D$N%T%/%;%k$N(B R, G, B $BCM$rF@$?$$$H$-$K$O!"(B
     $B#1%T%/%;%kEv$?$j(B 32 bit $B$N>l9g(B ACVC_GETR32, ACVC_GETG32, ACVC_GETB32 $B$r(B
     $BMQ$$!"(B16 bit $B$N>l9g(B ACVC_GETR16, ACVC_GETG16, ACVC_GETB16 $B$rMQ$$$^$9!#(B
     $BF@$i$l$k(B R, G, B $BCM$O!"#1%T%/%;%kEv$?$j(B 32 bit $B$N(B $B>l9g(B 0 $B0J>e(B 255 $B0J2<!"(B
     16 bit $B$N>l9g(B 0 $B0J>e(B 31 $B0J2<$G$9!#(B
  */

  printf ("P6\n");
  printf ("%d %d\n", x, y);

  if (c) {
    printf ("%d\n", 255);

    q = (unsigned long *) buf;

    for (i = x * y; i > 0; i --) {
      putchar (ACVC_GETR32 (* q));
      putchar (ACVC_GETG32 (* q));
      putchar (ACVC_GETB32 (* q));
      q ++;
    }
  } else {
    printf ("%d\n", 31);

    p = (unsigned short *) buf;

    for (i = x * y; i > 0; i --) {
      putchar (ACVC_GETR16 (* p));
      putchar (ACVC_GETG16 (* p));
      putchar (ACVC_GETB16 (* p));
      p ++;
    }
  }

  return 0;
}
