/*   ArgoCraft Video Capture Board    */
/*   Driver and Sample Applications   */
/* Version 1.32 for Linux and FreeBSD */
/* Copyright (C) 1996 ArgoCraft Corp. */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <fcntl.h>

#include "acvc.h"

#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>

#include <X11/Xaw/Simple.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Toggle.h>

#define pad(x) ((x) == 24 ? 32 : (x))
#define even(x) ((x) < 2 ? 2 : (x) / 2 * 2)
#define fivebits(x) ((x) < 0 ? 0 : (x) >= 32 ? 31 : (x))

extern void XtResizeWidget (Widget, Dimension, Dimension, Dimension);

static Widget top,
                form,
                  showWindow,
                  viewForm,
                    viewLabel, viewUp, viewDown, viewNum,
                  srcForm,
                    srcLabel, srcUp, srcDown, srcNum,
                  briForm,
                    briLabel, briUp, briDown, briNum,
                  conForm,
                    conLabel, conUp, conDown, conNum,
                  satForm,
                    satLabel, satUp, satDown, satNum,
                  otherForm,
                    halfToggle, monoToggle, quitButton;

static int fd = - 1, x = 200, y = 150, num, size;
static int half = 1, mono, view;
static unsigned char src = 0, bri = 192;
static signed char con = 64, sat = 64;

static void * framebuf;
static char strbuf [10];

static unsigned char dit [4] [4] [32768];

static const int err [4] [4] = {
  { 0, 6, 1, 7 },
  { 4, 2, 5, 3 },
  { 1, 7, 0, 6 },
  { 5, 3, 4, 2 }
};

static const int erg [4] [4] = {
  { 1, 3, 1, 3 },
  { 2, 0, 2, 0 },
  { 1, 3, 1, 3 },
  { 2, 0, 2, 0 }
};

static const int erb [4] [4] = {
  { 7, 1, 6, 0 },
  { 3, 5, 2, 4 },
  { 6, 0, 7, 1 },
  { 2, 4, 3, 5 }
};

static unsigned char coltbl [128];
static unsigned char monotbl [32];

static Display * disp;
static Screen * scr;
static Window win;
static int depth;
static Visual * vi;
static GC gc;
static Colormap cm;
static XColor col;
static XImage * img = NULL;

/**********************************************************************/

static int setview (int v) {

#define expand(x) (even ((x) * v * (half ? 2 : 1) / 100))

  if (ACVC_SETVIEW (fd, expand (x), expand (y))) {
    return - 1;
  }

  view = v;
  sprintf (strbuf, "%d", view);
  XtVaSetValues (viewNum, XtNlabel, strbuf, NULL);

  return 0;
}

static void setbri (unsigned char b) {
  if (ACVC_SETBRI (fd, b)) {
    perror ("xvctest2: SETBRI");
    exit (1);
  }

  bri = b;
  sprintf (strbuf, "%d", bri);
  XtVaSetValues (briNum, XtNlabel, strbuf, NULL);
}

static void setcon (signed char c) {
  if (ACVC_SETCON (fd, c)) {
    perror ("xvctest2: SETCON");
    exit (1);
  }

  con = c;
  sprintf (strbuf, "%d", con);
  XtVaSetValues (conNum, XtNlabel, strbuf, NULL);
}

static void setsat (signed char s) {
  if (ACVC_SETSAT (fd, s)) {
    perror ("xvctest2: SETSAT");
    exit (1);
  }

  sat = s;
  sprintf (strbuf, "%d", sat);
  XtVaSetValues (satNum, XtNlabel, strbuf, NULL);
}

static void setsrc (unsigned char s) {
  if (ACVC_SETSRC (fd, s)) {
    return;
  }

  src = s;
  sprintf (strbuf, "No.%d", src);
  XtVaSetValues (srcNum, XtNlabel, strbuf, NULL);
}

static void vcinit (void) {
  int d, i, lx, ly;

  if (fd >= 0) {
    if (close (fd)) {
      perror ("xvctest2: close");
      exit (1);
    }
  }

  if ((fd = open (half ? "/dev/acvc0" : "/dev/acvc", O_RDONLY)) < 0) {
    perror ("xvctest2: open");
    exit (1);
  }

  XtVaSetValues (halfToggle, XtNstate, half, NULL);

  switch (depth) {
  case 8:
  case 16:
    d = mono;
    break;
  case 24:
  default:
    d = 1;
    break;
  }

  if (half) {
    lx = 320;
    ly = 240;
  } else {
    lx = 640;
    ly = 480;
  }

  if (x > lx) {
    x = lx;
  }

  if (y > ly) {
    y = ly;
  }

  x = even (x);
  y = even (y);

  if (ACVC_SETSIZE (fd, x, y, d)) {
    perror ("xvctest2: SETSIZE");
    exit (1);
  }

  num = x * y;
  size = num * (d ? 4 : 2);
  XtResizeWidget (showWindow, x, y, 1);

  if (img) {
    XDestroyImage (img);
  }

  if (! (framebuf = XtMalloc (size))) {
    fprintf (stderr, "xvctest2: malloc failed\n");
    exit (1);
  }

  img = XCreateImage (disp, vi, depth, ZPixmap, 0,
		      framebuf, x, y, pad (depth), 0);

  if (! XInitImage (img)) {
    fprintf (stderr, "xvctest2: can't initialize image\n");
    exit (1);
  }

  if (img -> byte_order != LSBFirst) {
    fprintf (stderr, "xvctest2: byte order doesn't match\n");
    exit (1);
  }

  if (ACVC_SETMONO (fd, mono)) {
    perror ("xvctest2: SETMONO");
    exit (1);
  }

  XtVaSetValues (monoToggle, XtNstate, mono, NULL);

  for (i = 100; i > 0; i --) {
    if (! setview (i * 10)) {
      break;
    }
  }

  setview (-- i * 10);

  setbri (bri);
  setcon (con);
  setsat (sat);

  setsrc (src);
}

/**********************************************************************/

static void cbfun (Widget w, XtPointer client, XtPointer call) {
  if (w == viewUp) {
    setview (view + 10);
  } else if (w == viewDown) {
    setview (view - 10);
  } else if (w == srcUp) {
    setsrc ((src + 1) % 6);
  } else if (w == srcDown) {
    setsrc ((src + 5) % 6);
  } else if (w == briUp) {
    setbri (bri + 8);
  } else if (w == briDown) {
    setbri (bri - 8);
  } else if (w == conUp) {
    setcon (con + 8);
  } else if (w == conDown) {
    setcon (con - 8);
  } else if (w == satUp) {
    setsat (sat + 8);
  } else if (w == satDown) {
    setsat (sat - 8);
  } else if (w == monoToggle) {
    mono = ! mono;
    vcinit ();
  } else if (w == halfToggle) {
    half = ! half;
    vcinit ();
  } else if (w == quitButton) {
    exit (0);
  }
}

static void resize (Widget w, XtPointer clos, XEvent * ev, Boolean * cont) {
  x = ev -> xconfigurerequest . width;
  y = ev -> xconfigurerequest . height;

  vcinit ();

  * cont = TRUE;
}

/**********************************************************************/

int main (int argc, char ** argv) {
  unsigned char dtr [32], dtg [32], dtb [32];
  int i, j, k, m, n, r, g, b;
  unsigned short * ps;
  unsigned long * pl;
  unsigned char * pc, c;
  XEvent ev;

  top = XtInitialize (argv [0],
		      "XVCTest2",
		      NULL,
		      0,
		      & argc,
		      argv);

  if (argc > 1) {
    fprintf (stderr, "xvctest2: invalid option\n");
    exit (1);
  }

  form = XtVaCreateManagedWidget ("form",
				  formWidgetClass,
				  top,
				  NULL);

  briForm = XtVaCreateManagedWidget ("briForm",
				     formWidgetClass,
				     form,
				     XtNleft, XtChainLeft,
				     XtNright, XtChainLeft,
				     XtNtop, XtChainTop,
				     XtNbottom, XtChainTop,
				     NULL);
  
  briLabel = XtVaCreateManagedWidget ("briLabel",
				      labelWidgetClass,
				      briForm,
				      XtNborderWidth, 0,
				      XtNlabel, "MMMMM",
				      NULL);
  
  briNum = XtVaCreateManagedWidget ("briNum",
				    labelWidgetClass,
				    briForm,
				    XtNborderWidth, 0,
				    XtNlabel, "MMMM",
				    XtNfromHoriz, briLabel,
				    NULL);
  
  briUp = XtVaCreateManagedWidget ("briUp",
				   commandWidgetClass,
				   briForm,
				   XtNshapeStyle, XawShapeOval,
				   XtNlabel, "MMMM",
				   XtNfromVert, briLabel,
				   NULL);
  
  briDown = XtVaCreateManagedWidget ("briDown",
				     commandWidgetClass,
				     briForm,
				     XtNshapeStyle, XawShapeOval,
				     XtNlabel, "MMMM",
				     XtNfromHoriz, briUp,
				     XtNfromVert, briLabel,
				     NULL);
  
  conForm = XtVaCreateManagedWidget ("conForm",
				     formWidgetClass,
				     form,
				     XtNfromVert, briForm,
				     XtNleft, XtChainLeft,
				     XtNright, XtChainLeft,
				     XtNtop, XtChainTop,
				     XtNbottom, XtChainTop,
				     NULL);
  
  conLabel = XtVaCreateManagedWidget ("conLabel",
				      labelWidgetClass,
				      conForm,
				      XtNborderWidth, 0,
				      XtNlabel, "MMMMM",
				      NULL);
  
  conNum = XtVaCreateManagedWidget ("conNum",
				    labelWidgetClass,
				    conForm,
				    XtNborderWidth, 0,
				    XtNlabel, "MMMM",
				    XtNfromHoriz, conLabel,
				    NULL);
  
  conUp = XtVaCreateManagedWidget ("conUp",
				   commandWidgetClass,
				   conForm,
				   XtNshapeStyle, XawShapeOval,
				   XtNlabel, "MMMM",
				   XtNfromVert, conLabel,
				   NULL);
  
  conDown = XtVaCreateManagedWidget ("conDown",
				     commandWidgetClass,
				     conForm,
				     XtNshapeStyle, XawShapeOval,
				     XtNlabel, "MMMM",
				     XtNfromHoriz, conUp,
				     XtNfromVert, conLabel,
				     NULL);
  
  satForm = XtVaCreateManagedWidget ("satForm",
				     formWidgetClass,
				     form,
				     XtNfromVert, conForm,
				     XtNleft, XtChainLeft,
				     XtNright, XtChainLeft,
				     XtNtop, XtChainTop,
				     XtNbottom, XtChainTop,
				     NULL);
  
  satLabel = XtVaCreateManagedWidget ("satLabel",
				      labelWidgetClass,
				      satForm,
				      XtNborderWidth, 0,
				      XtNlabel, "MMMMM",
				      NULL);
  
  satNum = XtVaCreateManagedWidget ("satNum",
				    labelWidgetClass,
				    satForm,
				    XtNborderWidth, 0,
				    XtNlabel, "MMMM",
				    XtNfromHoriz, satLabel,
				    NULL);
  
  satUp = XtVaCreateManagedWidget ("satUp",
				   commandWidgetClass,
				   satForm,
				   XtNshapeStyle, XawShapeOval,
				   XtNlabel, "MMMM",
				   XtNfromVert, satLabel,
				   NULL);
  
  satDown = XtVaCreateManagedWidget ("satDown",
				     commandWidgetClass,
				     satForm,
				     XtNshapeStyle, XawShapeOval,
				     XtNlabel, "MMMM",
				     XtNfromHoriz, satUp,
				     XtNfromVert, satLabel,
				     NULL);
  
  viewForm = XtVaCreateManagedWidget ("viewForm",
				      formWidgetClass,
				      form,
				      XtNfromHoriz, briForm,
				      XtNleft, XtChainLeft,
				      XtNright, XtChainLeft,
				      XtNtop, XtChainTop,
				      XtNbottom, XtChainTop,
				      NULL);
  
  viewLabel = XtVaCreateManagedWidget ("viewLabel",
				       labelWidgetClass,
				       viewForm,
				       XtNborderWidth, 0,
				       XtNlabel, "MMMMM",
				       NULL);
  
  viewNum = XtVaCreateManagedWidget ("viewNum",
				     labelWidgetClass,
				     viewForm,
				     XtNborderWidth, 0,
				     XtNlabel, "MMMM",
				     XtNfromHoriz, viewLabel,
				     NULL);
  
  viewUp = XtVaCreateManagedWidget ("viewUp",
				    commandWidgetClass,
				    viewForm,
				    XtNshapeStyle, XawShapeOval,
				    XtNlabel, "MMMM",
				    XtNfromVert, viewLabel,
				    NULL);
  
  viewDown = XtVaCreateManagedWidget ("viewDown",
				      commandWidgetClass,
				      viewForm,
				      XtNshapeStyle, XawShapeOval,
				      XtNlabel, "MMMM",
				      XtNfromHoriz, viewUp,
				      XtNfromVert, viewLabel,
				      NULL);
  
  srcForm = XtVaCreateManagedWidget ("srcForm",
				     formWidgetClass,
				     form,
				     XtNfromHoriz, briForm,
				     XtNfromVert, viewForm,
				     XtNleft, XtChainLeft,
				     XtNright, XtChainLeft,
				     XtNtop, XtChainTop,
				     XtNbottom, XtChainTop,
				     NULL);
  
  srcLabel = XtVaCreateManagedWidget ("srcLabel",
				      labelWidgetClass,
				      srcForm,
				      XtNborderWidth, 0,
				      XtNlabel, "MMMMM",
				      NULL);
  
  srcNum = XtVaCreateManagedWidget ("srcNum",
				    labelWidgetClass,
				    srcForm,
				    XtNborderWidth, 0,
				    XtNlabel, "MMMM",
				    XtNfromHoriz, srcLabel,
				    NULL);
  
  srcUp = XtVaCreateManagedWidget ("srcUp",
				   commandWidgetClass,
				   srcForm,
				   XtNshapeStyle, XawShapeOval,
				   XtNlabel, "MMMM",
				   XtNfromVert, srcLabel,
				   NULL);
  
  srcDown = XtVaCreateManagedWidget ("srcDown",
				     commandWidgetClass,
				     srcForm,
				     XtNshapeStyle, XawShapeOval,
				     XtNlabel, "MMMM",
				     XtNfromHoriz, srcUp,
				     XtNfromVert, srcLabel,
				     NULL);
  
  otherForm = XtVaCreateManagedWidget ("otherForm",
				       formWidgetClass,
				       form,
				       XtNborderWidth, 0,
				       XtNfromHoriz, briForm,
				       XtNfromVert, srcForm,
				       XtNleft, XtChainLeft,
				       XtNright, XtChainLeft,
				       XtNtop, XtChainTop,
				       XtNbottom, XtChainTop,
				       NULL);
  
  monoToggle = XtVaCreateManagedWidget ("monoToggle",
					toggleWidgetClass,
					otherForm,
					XtNlabel, "MMMM",
					NULL);

  halfToggle = XtVaCreateManagedWidget ("halfToggle",
					toggleWidgetClass,
					otherForm,
					XtNlabel, "MMMM",
					XtNfromVert, monoToggle,
					NULL);

  quitButton = XtVaCreateManagedWidget ("quitButton",
					commandWidgetClass,
					otherForm,
					XtNshapeStyle, XawShapeOval,
					XtNlabel, "MMMM",
					XtNfromHoriz, halfToggle,
					XtNfromVert, monoToggle,
					NULL);

  showWindow = XtVaCreateManagedWidget ("showWindow",
					simpleWidgetClass,
					form,
					XtNwidth, x,
					XtNheight, y,
					XtNfromHoriz, viewForm,
					XtNleft, XtChainLeft,
					XtNtop, XtChainTop,
					NULL);

  XtAddCallback (viewUp, XtNcallback, cbfun, NULL);
  XtAddCallback (viewDown, XtNcallback, cbfun, NULL);
  XtAddCallback (srcUp, XtNcallback, cbfun, NULL);
  XtAddCallback (srcDown, XtNcallback, cbfun, NULL);
  XtAddCallback (briUp, XtNcallback, cbfun, NULL);
  XtAddCallback (briDown, XtNcallback, cbfun, NULL);
  XtAddCallback (conUp, XtNcallback, cbfun, NULL);
  XtAddCallback (conDown, XtNcallback, cbfun, NULL);
  XtAddCallback (satUp, XtNcallback, cbfun, NULL);
  XtAddCallback (satDown, XtNcallback, cbfun, NULL);
  XtAddCallback (monoToggle, XtNcallback, cbfun, NULL);
  XtAddCallback (halfToggle, XtNcallback, cbfun, NULL);
  XtAddCallback (quitButton, XtNcallback, cbfun, NULL);

  XtRealizeWidget (top);

  XtAddEventHandler (showWindow, StructureNotifyMask, FALSE, resize, NULL);

  XtVaSetValues (briLabel, XtNlabel, "Bri:", NULL);
  XtVaSetValues (briUp, XtNlabel, "Up", NULL);
  XtVaSetValues (briDown, XtNlabel, "Down", NULL);

  XtVaSetValues (conLabel, XtNlabel, "Con:", NULL);
  XtVaSetValues (conUp, XtNlabel, "Up", NULL);
  XtVaSetValues (conDown, XtNlabel, "Down", NULL);

  XtVaSetValues (satLabel, XtNlabel, "Sat:", NULL);
  XtVaSetValues (satUp, XtNlabel, "Up", NULL);
  XtVaSetValues (satDown, XtNlabel, "Down", NULL);

  XtVaSetValues (viewLabel, XtNlabel, "View:", NULL);
  XtVaSetValues (viewUp, XtNlabel, "Up", NULL);
  XtVaSetValues (viewDown, XtNlabel, "Down", NULL);

  XtVaSetValues (srcLabel, XtNlabel, "Src:", NULL);
  XtVaSetValues (srcUp, XtNlabel, "Up", NULL);
  XtVaSetValues (srcDown, XtNlabel, "Down", NULL);

  XtVaSetValues (monoToggle, XtNlabel, "Mono", NULL);
  XtVaSetValues (halfToggle, XtNlabel, "Half", NULL);
  XtVaSetValues (quitButton, XtNlabel, "Quit", NULL);

  /**********************************************************************/

  disp = XtDisplay (showWindow);
  scr = XtScreen (showWindow);
  win = XtWindow (showWindow);

  depth = DefaultDepthOfScreen (scr);
  vi = DefaultVisualOfScreen (scr);
  gc = DefaultGCOfScreen (scr);
  cm = DefaultColormapOfScreen (scr);

  switch (depth) {
  case 8:
    for (m = 0; m < 4; m ++) {
      for (n = 0; n < 4; n ++) {
	for (i = 0; i < 32; i ++) {
	  r = i + err [m] [n];
	  g = i + erg [m] [n];
	  b = i + erb [m] [n];
	  
	  r = fivebits (r) >> 3;
	  g = fivebits (g) >> 2;
	  b = fivebits (b) >> 3;
	  
	  dtr [i] = r << 5;
	  dtg [i] = g << 2;
	  dtb [i] = b;
	}
	
	for (i = 0; i < 32; i ++) {
	  for (j = 0; j < 32; j ++) {
	    for (k = 0; k < 32; k ++) {
	      dit [m] [n] [i << 10 | j << 5 | k] = dtr [i] | dtg [j] | dtb [k];
	    }
	  }
	}
      }
    }
    
    for (i = 0; i < 4; i ++) {
      for (j = 0; j < 8; j ++) {
	for (k = 0; k < 4; k ++) {
	  col . red = i << 14;
	  col . green = j << 13;
	  col . blue = k << 14;

	  XAllocColor (disp, cm, & col);

	  coltbl [i << 5 | j << 2 | k] = col . pixel;
	}
      }
    }

    for (i = 0; i < 32; i ++) {
      col . red = col . green = col . blue = i << 11;
      XAllocColor (disp, cm, & col);
      monotbl [i] = col . pixel;
    }

    break;
  case 16:
    if (vi -> red_mask != 0xf800 ||
	vi -> green_mask != 0x07e0 ||
	vi -> blue_mask != 0x001f) {
      fprintf (stderr, "xvctest2: visual doesn't match\n");
      exit (1);
    }

    break;
  case 24:
    if (vi -> red_mask != 0xff0000 ||
	vi -> green_mask != 0x00ff00 ||
	vi -> blue_mask != 0x0000ff) {
      fprintf (stderr, "xvctest2: visual doesn't match\n");
      exit (1);
    }

    break;
  default:
    fprintf (stderr, "xvctest2: visual doesn't match\n");
    exit (1);
  }

  /**********************************************************************/

  vcinit ();

  if (nice (10)) {
    perror ("xvctest2: nice");
    exit (1);
  }

  while (1) {
    while (XtPending ()) {
      XtNextEvent (& ev);
      XtDispatchEvent (& ev);
    }

    if ((n = read (fd, framebuf, size)) < 0) {
      perror ("xvctest2: read");
      exit (1);
    }
    
    if (ACVC_CAPSTART (fd)) {
      perror ("xvctest2: CAPSTART");
      exit (1);
    }
    
    if (n != size) {
      fprintf (stderr, "xvctest2: read failed\n");
      exit (1);
    }

    switch (depth) {
    case 8:
      if (mono) {
	pl = framebuf;
	pc = framebuf;

	for (i = 0; i < num; i ++) {
	  * pc ++ = monotbl [* (unsigned char *) pl ++ >> 3];
	}
      } else {
	ps = framebuf;
	pc = framebuf;
	
	for (j = 0; j < y; j ++) {
	  for (i = 0; i < x; i ++) {
	    * pc ++ = coltbl [dit [i & 3] [j & 3] [* ps ++]];
	  }
	}
      }

      break;
    case 16:
      if (mono) {
	ps = framebuf;
	pl = framebuf;

	for (i = num; i > 0; i --) {
	  c = * (unsigned char *) pl ++;
	  * ps ++ = (c >> 3 << 11) | (c >> 2 << 5) | (c >> 3);
	}
      } else {
	ps = framebuf;
	
	for (i = num; i > 0; i --) {
	  * ps = (* ps & 0x001f) | (* ps & 0xffe0) << 1;
	  ps ++;
	}
      }

      break;
    case 24:
    default:
      break;
    }

    XPutImage (disp, win, gc, img, 0, 0, 0, 0, x, y);
  }

  return 0;
}
