/*   ArgoCraft Video Capture Board    */
/*   Driver and Sample Applications   */
/* Version 1.32 for Linux and FreeBSD */
/* Copyright (C) 1996 ArgoCraft Corp. */

#include "acvc.h"

#if NACVC > 0

#include <sys/errno.h>
#include <sys/exec.h>
#include <sys/mman.h>
#include <sys/param.h>
#include <sys/proc.h>
#include <sys/sysent.h>
#include <sys/systm.h>
#include <sys/types.h>
#include <sys/conf.h>
#include <sys/fcntl.h>
#include <sys/kernel.h>
#include <sys/malloc.h>

#include <vm/vm.h>
#include <vm/vm_kern.h>
#include <vm/vm_extern.h>
#include <vm/vm_prot.h>
#include <vm/vm_page.h>

#include <i386/isa/icu.h>
#include <i386/isa/isa_device.h>

#include <pci/pcireg.h>
#include <pci/pcivar.h>
#include <pci/pcibus.h>

#include <machine/clock.h>
#include <machine/cpu.h>
#include <machine/ioctl_acvc.h>

#define ACVC_MAJOR 20
#define ACVC_BUFSIZE_ ((ACVC_BUFSIZE / 2 / PAGE_SIZE + 1) * PAGE_SIZE)

#define LOBYTE(w) ((unsigned char) (unsigned short) (w))
#define HIBYTE(w) ((unsigned char) ((unsigned short) (w) >> 8))

#define LOWORD(dw) ((unsigned short) (unsigned long) (dw))
#define HIWORD(dw) ((unsigned short) ((unsigned long) (dw) >> 16))

#define MAXMEMTRYNUM 16
#define MAXRETRYNUM 8

#define MAX_WIDTH 640
#define MAX_HEIGHT 240

static int init = 0, busy = 0, pending = 0;
static unsigned long rest = 0;

static unsigned long bufvirt [2] = { 0, 0 }, bufphys [2] = { 0, 0 };
static unsigned long regvirt = 0, regphys = 0;

static unsigned short insize [2] = { MAX_WIDTH, MAX_HEIGHT };
static unsigned short exsize [2] = { MAX_WIDTH, MAX_HEIGHT };

static unsigned long fieldsize = 0;
static unsigned long framesize = 0;

static int incolor = 1; /* 0 == rgb16, 1 == rgb32 */
static int excolor = 0; /* 0 == color, 1 == mono */

#define DEF_BRI 0x8b
#define DEF_CON 0x46
#define DEF_SAT 0x40

static unsigned char src = 0;

static unsigned char bri = 0x8b;
static signed char con = 0x46;
static signed char sat = 0x40;

#define NUM_REG_7186 0x11

static unsigned char reg7186 [NUM_REG_7186] = {
  0x12, 0x40, 0x40, 0x10, 0x85, 0x78, 0x78, 0x10,
  0x80, 0x03, 0x80, 0x00, 0x00, 0xff, 0x00, 0x00,
  0x0f
};

#define NUM_REG_7110 0x35

static unsigned char reg7110 [NUM_REG_7110] = { 
  0x4c, 0x3c, 0x0d, 0xef, 0xbd, 0xf0, 0x00, 0x00,
  0xf8, 0xf8, 0x60, 0x60, 0x00, 0x06, 0x18, 0x90,
  0x00, 0x2c, 0x40, 0x46, 0x42, 0x1a, 0xff, 0xda,
  0xf0, 0x8b, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
  0xd9, 0x16, 0x40, 0x41, 0x80, 0x41, 0x80, 0x4f,
  0xfe, 0x01, 0xcf, 0x0f, 0x03, 0x01, 0x81, 0x03,
  0x44, 0x75, 0x01, 0x8c, 0x03
};

#define NUM_SRC 6
#define NUM_SEL 8

static const unsigned char srcsel [NUM_SEL] = {
  0x06, 0x20, 0x21, 0x22, 0x2c, 0x30, 0x31, 0x21
};

static const unsigned char srcval [NUM_SRC] [NUM_SEL] = {
  { 0x00, 0xd9, 0x17, 0x40, 0x03, 0x44, 0x75, 0x16 }, /* mode 0 */
  { 0x00, 0xd8, 0x17, 0x40, 0x03, 0x44, 0x75, 0x16 }, /* mode 1 */
  { 0x00, 0xba, 0x07, 0x91, 0x03, 0x60, 0xb5, 0x05 }, /* mode 2 */
  { 0x00, 0xb8, 0x07, 0x91, 0x03, 0x60, 0xb5, 0x05 }, /* mode 3 */
  { 0x00, 0x7c, 0x07, 0xd2, 0x83, 0x60, 0xb5, 0x03 }, /* mode 4 */
  { 0x00, 0x78, 0x07, 0xd2, 0x83, 0x60, 0xb5, 0x03 }, /* mode 5 */
};

static void longwait (void) {
  char foobar;

  if (init) {
    tsleep (& foobar, curpriority, "acvc", hz / 10);
  } else {
    DELAY (1000 * 1000 / 10);
  }
}

static void shortwait (void) {
  char foobar;

  if (init) {
    tsleep (& foobar, curpriority, "acvc", hz / 100);
  } else {
    DELAY (1000 * 1000 / 100);
  }
}

static unsigned long allocbuf (void) {
  unsigned long addr;

  addr = vm_page_alloc_contig (ACVC_BUFSIZE_, 0x100000, 0xffffffff, 1 << 24);

  if (addr) {
    return addr;
  }

  addr = vm_page_alloc_contig (ACVC_BUFSIZE_, 0x100000, 0xffffffff, PAGE_SIZE);

  if (addr) {
    if (bootverbose) {
      printf ("acvc: buffer not aligned\n");
    }
    return addr;
  }

  return 0;
}

static void freebuf (unsigned long addr) {
  if (addr != 0) {
    /* should free frame buffer */
  }
}

static unsigned long read7116 (unsigned char addr) {
  return ((volatile unsigned long *) regvirt) [addr / 4];
}

static void write7116 (unsigned char addr, unsigned long val) {
  ((volatile unsigned long *) regvirt) [addr / 4] = val;
}

static void write7186 (unsigned char addr, unsigned char val) {
  unsigned long status, count = 0, timeout = 10;

  write7116 (0x64, 0x01b80000 | addr << 8 | val);
  reg7186 [addr] = val;

 retry:

  while (read7116 (0x64) & 0x01000000) {
    shortwait ();

    if (! timeout --) {
      printf ("acvc: write7186 timed out\n");
      return;
    }
  }

  if ((status = read7116 (0x60)) & 0x00000020) {
    write7116 (0x60, status);

    if (++ count > MAXRETRYNUM) {
      printf ("acvc: too many retries\n");
      return;
    }

    /* printf ("acvc: retrying... (%ld)\n", count); */
    goto retry;
  }
}

static void write7110 (unsigned char addr, unsigned char val) {
  unsigned long status, count = 0, timeout = 10;

  write7116 (0x64, 0x019c0000 | addr << 8 | val);
  reg7110 [addr] = val;

 retry:

  while (read7116 (0x64) & 0x01000000) {
    shortwait ();

    if (! timeout --) {
      printf ("acvc: write7110 timed out\n");
      return;
    }
  }

  if ((status = read7116 (0x60)) & 0x00000020) {
    write7116 (0x60, status);

    if (++ count > MAXRETRYNUM) {
      printf ("acvc: too many retries\n");
      return;
    }

    /* printf ("acvc: retrying... (%ld)\n", count); */
    goto retry;
  }
}

static int setin (unsigned short x, unsigned short y, int c) {
  unsigned long size;

  if (! c && excolor) {
    return - 1;
  }

  if (! x || ! y) {
    return - 1;
  }

  if (x > exsize [0] || y > exsize [1]) {
    return - 1;
  }

  if (! c && x % 2) {
    return - 1;
  }

  size = x * y * (c ? 4 : 2);

  fieldsize = x * (c ? 4 : 2);
  framesize = size;

  rest = framesize;

  insize [0] = x;
  insize [1] = y;
  incolor = c;

  if (c) {
    write7116 (0x30, 0x39393900);
    write7116 (0x34, 0x39393900);
    write7186 (0x00, reg7186 [0x00] | 0x02);
  } else {
    write7116 (0x30, 0xeeeeee01);
    write7116 (0x34, 0xeeeeee01);
    write7186 (0x00, reg7186 [0x00] & 0xfd);
  }

  write7116 (0x8c, bufphys [0] + size);
  write7116 (0x90, bufphys [1] + size);

  write7186 (0x01, LOBYTE (x));
  write7186 (0x04, HIBYTE (x) | (reg7186 [0x04] & 0xfc));

  write7186 (0x05, LOBYTE (y));
  write7186 (0x08, HIBYTE (y) | (reg7186 [0x08] & 0xfc));

  return 0;
}

static int setexsize (unsigned short x, unsigned short y) {
  unsigned short ox, oy;

  if (x > MAX_WIDTH || y > MAX_HEIGHT) {
    return - 1;
  }

  if (x < insize [0] || y < insize [1]) {
    return - 1;
  }

  exsize [0] = x;
  exsize [1] = y;

  write7186 (0x02, LOBYTE (x));
  write7186 (0x04, HIBYTE (x) << 2 | (reg7186 [0x04] & 0xf3));

  write7186 (0x06, LOBYTE (y));
  write7186 (0x08, HIBYTE (y) << 2 | (reg7186 [0x08] & 0xf3));

  ox = 0x10 + ((MAX_WIDTH - x) / 2);
  oy = 0x10 + ((MAX_HEIGHT - y) / 2);
  
  write7186 (0x03, LOBYTE (ox));

  if (HIBYTE (ox) == 0) {
    write7186 (0x04, reg7186 [0x04] & 0xef);
  } else {
    write7186 (0x04, reg7186 [0x04] | 0x10);
  }

  write7186 (0x07, LOBYTE (oy));

  if (HIBYTE (oy) == 0) {
    write7186 (0x08, reg7186 [0x08] & 0xef);
  } else {
    write7186 (0x08, reg7186 [0x08] | 0x10);
  }

  return 0;
}

static int setexcolor (int e) {
  if (e & ! incolor) {
    return - 1;
  }

  excolor = e;

  if (e) {
    write7186 (0x00, reg7186 [0x00] | 0x01);
    write7186 (0x10, reg7186 [0x10] | 0x10);
  } else {
    write7186 (0x00, reg7186 [0x00] & 0xfe);
    write7186 (0x10, reg7186 [0x10] & 0xef);
  }

  return 0;
}

static int setsrc (unsigned char s) {
  int i;

  if (s >= NUM_SRC) {
    return - 1;
  }

  src = s;

  for (i = 0; i < NUM_SEL; i ++) {
    write7110 (srcsel [i], srcval [s] [i]);
  }

  return 0;
}

static void setbri (unsigned char b) {
  bri = b;
  write7110 (0x19, b);
}

static void setcon (signed char c) {
  con = c;
  write7110 (0x13, c);
}

static void setsat (signed char s) {
  sat = s;
  write7110 (0x12, s);
}

static void init7116 (void) {
  /* initialization */

  write7116 (0x60, 0);
  write7116 (0x40, 0);
  
  longwait ();
  
  write7116 (0x40, 0x00000040);
  
  longwait ();
  
  write7116 (0x00, 0x00000004);
  write7116 (0x04, 0x00000004);
  write7116 (0x08, 0x00000004);
  write7116 (0x0c, 0x00000004);
  write7116 (0x10, 0x00000004);
  write7116 (0x14, 0x00000004);
  write7116 (0x8c, 0);
  write7116 (0x90, 0);
  write7116 (0x5c, 0x80404020);
  write7116 (0x40, 0x000080c0);
  
  longwait ();
  
  write7116 (0x40, 0x00008040);
  
  longwait ();
  
  /* DMA */

  write7116 (0x00, bufphys [0]);
  write7116 (0x04, 0);
  write7116 (0x08, 0);
  write7116 (0x0c, bufphys [1]);
  write7116 (0x10, 0);
  write7116 (0x14, 0);
  
  /* Stride */

  write7116 (0x18, 0);
  write7116 (0x1c, 0);
  write7116 (0x20, 0);
  write7116 (0x24, 0);
  write7116 (0x28, 0);
  write7116 (0x2c, 0);

  /* Capture Control */

  write7116 (0x38, 0x00200020);
  write7116 (0x3c, 0x00000003);
  write7116 (0x40, 0x000000c0);
  write7116 (0x44, 0);
  write7116 (0x48, 0);
  write7116 (0x4c, 0x00000001);
  write7116 (0x50, 0x00000001);
  write7116 (0x54, 0);
  write7116 (0x58, 0x0005007c);
  
  /* I2C */

  write7116 (0x5c, 0x461e1e0f);
  write7116 (0x60, 0);
  write7116 (0x64, 0);
  write7116 (0x68, 0);
  write7116 (0x6c, 0);
  write7116 (0x70, 0);
  write7116 (0x74, 0);
  write7116 (0x78, 0);
  write7116 (0x7c, 0);
  write7116 (0x80, 0);
  write7116 (0x84, 0);
  write7116 (0x88, 0);
  
  write7116 (0x40, 0x00008ff0);
}

static void init7186 (void) {
  unsigned long addr = NUM_REG_7186;

  while (addr --) {
    write7186 (addr, reg7186 [addr]);
  }
}

static void init7110 (void) {
  unsigned long addr = NUM_REG_7110;

  while (addr --) {
    write7110 (addr, reg7110 [addr]);
  }
}

static int iscap (void) {
  return (read7116 (0x40) & 0x0000000c) ? 1 : 0;
}

static void capstart (int which) {
  const unsigned long val [3] = { 0x00008ffc, 0x00008ff4, 0x00008ff8 };

  write7116 (0x40, val [which]);
  pending = 1;
  rest = framesize;
}

static int capwait (int which) {
  const unsigned long val [3] = { 0x00000030, 0x00000010, 0x00000020 };
  unsigned long status, count = 0, timeout;

 retry:

  if (! pending) {
    return 0;
  }

  pending = 0;

  /* wait for capture */

  timeout = 100;

  while (iscap ()) {
    shortwait ();

    if (! timeout --) {
      printf ("acvc: capture timed out\n");
      return - 1;
    }
  }

  /* wait for transfer */

  timeout = 100;

  while (~ read7116 (0x40) & val [which]) {
    shortwait ();

    if (! timeout --) {
      printf ("acvc: transfer timed out\n");
      return - 1;
    }
  }

  /* check up errors */

  if ((status = read7116 (0x40)) & 0x00000f00) {
    write7116 (0x40, status);

    if (status & 0x00000400) {
      printf ("acvc: address error (even)\n");
    }
    
    if (status & 0x00000800) {
      printf ("acvc: address error (odd)\n");
    }

    if (status & 0x00000c00) {
      return - 1;
    }

    if (status & 0x00000100) {
      /* printf ("acvc: field corrupt (even)\n"); */
    }

    if (status & 0x00000200) {
      /* printf ("acvc: field corrupt (odd)\n"); */
    }

    if (status & 0x00000300) {
      if (++ count > MAXRETRYNUM) {
	printf ("acvc: too many retries\n");
	return - 1;
      }
      
      /* printf ("acvc: retrying... (%d)\n", count); */

      capstart (which);
      goto retry;
    }
  }

  return 0;
}

static int acvc_read (dev_t dev, struct uio * uio, int flat) {
  char * p, * q;
  int len, err, i;

  if (minor (dev) > 2) {
    return ENXIO;
  }

  if (minor (dev)) {
    if ((len = min (rest, uio -> uio_resid)) == 0) {
      return 0;
    }
    
    if (capwait (minor (dev))) {
      return EIO;
    }

    err = uiomove ((void *) (bufvirt [minor (dev) - 1] + framesize - rest),
		   len, uio);

    if (err) {
      return err;
    }

    rest -= len;

    return err;
  }

  len = uio -> uio_resid;

  if (len < framesize * 2) {
    return EINVAL;
  }

  p = (char *) bufvirt [0];
  q = (char *) bufvirt [1];

  if (capwait (minor (dev))) {
    return EIO;
  }

  for (i = 0; i < insize [1]; i ++) {
    if ((err = uiomove (p, fieldsize, uio))) {
      return err;
    }

    p += fieldsize;

    if ((err = uiomove (q, fieldsize, uio))) {
      return err;
    }

    q += fieldsize;
  }

  return 0;
}

static int acvc_ioctl (dev_t dev, int cmd, caddr_t arg,
		       int flag, struct proc * p) {
  unsigned long val;
  
  switch (cmd) {
  case ACVC_ISCAP:
    {
      if (arg == NULL) {
	return EINVAL;
      }

      * (int *) arg = iscap ();

      return 0;
    }
  case ACVC_START:
    if (capwait (minor (dev))) {
      return EIO;
    }

    capstart (minor (dev));

    return 0;
  case ACVC_WAIT:
    if (capwait (minor (dev))) {
      return EIO;
    }

    return 0;
  case ACVC_SIZE:
    {
      int c;

      if (arg == NULL) {
	return EINVAL;
      }

      val = * (unsigned long *) arg;

      c = (val & 0x80000000) ? 1 : 0;
      val &= 0x7fffffff;

      if (capwait (minor (dev))) {
	return EIO;
      }

      if (minor (dev)) {
	if (setin (HIWORD (val), LOWORD (val), c)) {
	  return EINVAL;
	}
      } else {
	if (LOWORD (val) % 2) {
	  return EINVAL;
	}

	if (setin (HIWORD (val), LOWORD (val) / 2, c)) {
	  return EINVAL;
	}
      }

      return 0;
    }
  case ACVC_VIEW:
    if (arg == NULL) {
      return EINVAL;
    }

    val = * (unsigned long *) arg;

    if (capwait (minor (dev))) {
      return EIO;
    }

    if (LOWORD (val) % 2) {
      return EINVAL;
    }

    if (setexsize (HIWORD (val), LOWORD (val) / 2)) {
      return EINVAL;
    }

    return 0;
  case ACVC_MONO:
    if (arg == NULL) {
      return EINVAL;
    }

    val = * (unsigned long *) arg;

    if (capwait (minor (dev))) {
      return EIO;
    }

    if (setexcolor (val)) {
      return EINVAL;
    }

    return 0;
  case ACVC_SRC:
    if (arg == NULL) {
      return EINVAL;
    }

    val = * (unsigned long *) arg;

    if (capwait (minor (dev))) {
      return EIO;
    }

    if (setsrc (val)) {
      return EINVAL;
    }

    return 0;
  case ACVC_BRI:
    if (arg == NULL) {
      return EINVAL;
    }

    val = * (unsigned long *) arg;

    if (capwait (minor (dev))) {
      return EIO;
    }

    if (val & 0xffffff00) {
      return EINVAL;
    }

    setbri ((unsigned char) val);

    return 0;
  case ACVC_CON:
    if (arg == NULL) {
      return EINVAL;
    }

    val = * (unsigned long *) arg;

    if (capwait (minor (dev))) {
      return EIO;
    }

    if (val & 0xffffff00) {
      return EINVAL;
    }

    setcon ((signed char) (unsigned char) val);

    return 0;
  case ACVC_SAT:
    if (arg == NULL) {
      return EINVAL;
    }

    val = * (unsigned long *) arg;

    if (capwait (minor (dev))) {
      return EIO;
    }

    if (val & 0xffffff00) {
      return EINVAL;
    }

    setsat ((signed char) (unsigned char) val);

    return 0;
  default:
    return EINVAL;
  }
}

static int acvc_mmap (dev_t dev, int offset, int nprot) {
  if (minor (dev) == 0 || minor (dev) > 2) {
    return - 1;
  }

  if (nprot & PROT_EXEC) {
    return - 1;
  }

  if (offset >= ACVC_BUFSIZE_) {
    return - 1;
  }

  return i386_btop (bufphys [minor (dev) - 1] + offset);
}

static int acvc_open (dev_t dev, int flag, int mode, struct proc * p) {
  if (flag & O_NONBLOCK) {
    return ENODEV;
  }

  if (busy) {
    return EBUSY;
  }

  busy = 1;
  pending = 0;

  setin (MAX_WIDTH / (minor (dev) ? 2 : 1), MAX_HEIGHT, 1);
  setexsize (MAX_WIDTH, MAX_HEIGHT);
  setexcolor (0);

  setsrc (0);

  setbri (DEF_BRI);
  setcon (DEF_CON);
  setsat (DEF_SAT);

  capstart (minor (dev));

  return 0;
}

static int acvc_close (dev_t dev , int flag , int mode , struct proc * p) {
  capwait (minor (dev));
  busy = 0;
  return 0;
}

static struct cdevsw cdevsw_acvc = {
  acvc_open,
  acvc_close,
  acvc_read,
  nowrite,
  acvc_ioctl,
  nostop,
  noreset,
  nodevtotty,
  noselect,
  acvc_mmap,
  nostrategy,
  "acvc",
  NULL,
  - 1
};

static char * acvc_probe (pcici_t tag, pcidi_t type) {
  if (type == 0x12238086) {
    return ("Philips SAA7116");
  } else {
    return NULL;
  }
}

static void acvc_attach (pcici_t tag, int unit) {
  dev_t dev;
  int i;

  printf ("acvc: version 1.32\n");

  if (hz < 100) {
    printf ("acvc: hz too small\n");
    return;
  }

  if (unit > 0) {
    printf ("acvc: invalid unit number\n");
    return;
  }

  /* allocate frame buffer */

  if ((bufvirt [0] = allocbuf ()) == 0) {
    printf ("acvc: can't allocate frame buffer (even)\n");
    return;
  }

  if ((bufvirt [1] = allocbuf ()) == 0) {
    freebuf (bufvirt [0]);
    printf ("acvc: can't allocate frame buffer (odd)\n");
    return;
  }

  bufphys [0] = vtophys (bufvirt [0]);
  bufphys [1] = vtophys (bufvirt [1]);

  if (bufphys [0] == 0 || bufphys [1] == 0) {
    freebuf (bufvirt [0]);
    freebuf (bufvirt [1]);
    printf ("acvc: vtophys failed\n");
    return;
  }

  if (bootverbose) {
    printf ("acvc: frame buffer virtual address (even) = %lx\n", bufvirt [0]);
    printf ("acvc: frame buffer virtual address (odd) = %lx\n", bufvirt [1]);
    printf ("acvc: frame buffer physical address (even) = %lx\n", bufphys [0]);
    printf ("acvc: frame buffer physical address (odd) = %lx\n", bufphys [1]);
  }

  /* enable master & memory */

  pci_conf_write (tag, PCI_COMMAND_STATUS_REG,
		  pci_conf_read (tag, PCI_COMMAND_STATUS_REG) |
		  (PCI_COMMAND_MASTER_ENABLE & PCI_COMMAND_MEM_ENABLE));

  /* map memory registers */

  if (pci_map_mem (tag, PCI_MAP_REG_START,
		   (vm_offset_t *) & regvirt,
		   (vm_offset_t *) & regphys) == 0) {
    printf ("acvc: mmap failed\n");
    freebuf (bufvirt [0]);
    freebuf (bufvirt [1]);
    return;
  }

  if (regvirt == 0) {
    freebuf (bufvirt [0]);
    freebuf (bufvirt [1]);
    printf ("acvc: mmap failed\n");
    return;
  }

  if (bootverbose) {
    printf ("acvc: memory register physical address = %lx\n", regphys);
    printf ("acvc: memory register virtual address = %lx\n", regvirt);
  }

  /* initialize registers */

  printf ("acvc: initializing...\n");

  init7116 ();

  if (read7116 (0) != bufphys [0]) {
    freebuf (bufvirt [0]);
    freebuf (bufvirt [1]);
    /* should unmap memory registers (?) */
    printf ("acvc: init7116 failed\n");
    return;
  }

  init7186 ();
  init7110 ();

  setin (MAX_WIDTH, MAX_HEIGHT, 1);
  setexsize (MAX_WIDTH, MAX_HEIGHT);
  setexcolor (0);

  setsrc (0);

  setbri (DEF_BRI);
  setcon (DEF_CON);
  setsat (DEF_SAT);

  /* test capture */

  if (bootverbose) {
    printf ("acvc: test capture started\n");
  }

  capstart (0);

  if (capwait (0)) {
    freebuf (bufvirt [0]);
    freebuf (bufvirt [1]);
    /* should unmap memory registers (?) */
    printf ("acvc: test capture failed!\n");
    return;
  }

  if (bootverbose) {
    printf ("acvc: test capture succeeded\n");
  }

  /* register device */

  for (i = 0; i <= 2; i ++) {
    dev = makedev (ACVC_MAJOR, i);
    cdevsw_add (& dev, & cdevsw_acvc, NULL);
  }

  printf ("acvc: driver installed\n");

  init = 1;
}

static unsigned long acvc_count;

static struct pci_device acvc_device = {
  "acvc",
  acvc_probe,
  acvc_attach,
  & acvc_count
};

DATA_SET (pcidevice_set, acvc_device);

#endif /* NACVC > 0 */
