/*   ArgoCraft Video Capture Board    */
/*   Driver and Sample Applications   */
/* Version 1.32 for Linux and FreeBSD */
/* Copyright (C) 1996 ArgoCraft Corp. */

#ifndef ACVC_H
#define ACVC_H

#include <sys/ioctl.h>

#define ACVC_BUFSIZE (640 * 480 * 4)

#define ACVC_ID 0xac

#define ACVC_ISCAP	(_IOR (ACVC_ID, 0, sizeof (int)))
#define ACVC_START	(_IO (ACVC_ID, 1))
#define ACVC_WAIT	(_IO (ACVC_ID, 2))
#define ACVC_SIZE	(_IOW (ACVC_ID, 3, sizeof (unsigned long)))
#define ACVC_VIEW	(_IOW (ACVC_ID, 4, sizeof (unsigned long)))
#define ACVC_MONO	(_IOW (ACVC_ID, 5, sizeof (unsigned long)))
#define ACVC_SRC	(_IOW (ACVC_ID, 6, sizeof (unsigned long)))
#define ACVC_BRI	(_IOW (ACVC_ID, 7, sizeof (unsigned long)))
#define ACVC_CON	(_IOW (ACVC_ID, 8, sizeof (unsigned long)))
#define ACVC_SAT	(_IOW (ACVC_ID, 9, sizeof (unsigned long)))

#ifndef KERNEL

#define ACVC_ISCAPTURING(fd,p)	(ioctl ((fd), ACVC_ISCAP, (int *) (p)))

#define ACVC_CAPSTART(fd)	(ioctl ((fd), ACVC_START))
#define ACVC_CAPWAIT(fd)	(ioctl ((fd), ACVC_WAIT))

static unsigned long acvc_arg = 0;

#define ACVC_SETSIZE(fd,x,y,c) ( \
  acvc_arg = (unsigned long) (((unsigned short) (x) << 16) | \
			      ((unsigned short) (y)) | \
			      ((c) ? 0x80000000 : 0x00000000)), \
  ioctl ((fd), ACVC_SIZE, & acvc_arg) \
)

#define ACVC_SETVIEW(fd,x,y) ( \
  acvc_arg = (unsigned long) (((unsigned short) (x) << 16) | \
			      ((unsigned short) (y))), \
  ioctl ((fd), ACVC_VIEW, & acvc_arg) \
)

#define ACVC_SETMONO(fd,m) ( \
  acvc_arg = (unsigned long) (int) (m), \
  ioctl ((fd), ACVC_MONO, & acvc_arg) \
)

#define ACVC_SETSRC(fd,src) ( \
  acvc_arg = (unsigned long) (unsigned char) (src), \
  ioctl ((fd), ACVC_SRC, & acvc_arg) \
)

#define ACVC_SETBRI(fd,bri) ( \
  acvc_arg = (unsigned long) (unsigned char) (bri), \
  ioctl ((fd), ACVC_BRI, & acvc_arg) \
)

#define ACVC_SETCON(fd,con) ( \
  acvc_arg = (unsigned long) (unsigned char) (signed char) (con), \
  ioctl ((fd), ACVC_CON, & acvc_arg) \
)

#define ACVC_SETSAT(fd,sat) ( \
  acvc_arg = (unsigned long) (unsigned char) (signed char) (sat), \
  ioctl ((fd), ACVC_SAT, & acvc_arg) \
)

#endif /* KERNEL */

#define ACVC_GETR32(p) ((unsigned char) ((0x00ff0000 & (p)) >> 16))
#define ACVC_GETG32(p) ((unsigned char) ((0x0000ff00 & (p)) >> 8))
#define ACVC_GETB32(p) ((unsigned char) (0x000000ff & (p)))

#define ACVC_GETR16(p) ((unsigned char) ((0x7c00 & (p)) >> 10))
#define ACVC_GETG16(p) ((unsigned char) ((0x03e0 & (p)) >> 5))
#define ACVC_GETB16(p) ((unsigned char) (0x001f & (p)))

#endif /* ACVC_H */
