/*   ArgoCraft Video Capture Board    */
/*   Driver and Sample Applications   */
/* Version 1.32 for Linux and FreeBSD */
/* Copyright (C) 1996 ArgoCraft Corp. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include "acvc.h"

#define NUM 16

#define WIDTH 640
#define HEIGHT 480

int main (int argc, char ** argv) {
  unsigned short buf [WIDTH * HEIGHT], * p;
  int src1, src2, fd, i, j, k, n, c;
  char * name;
  FILE * fp;

  if (argc < 4) {
    fprintf (stderr, "usage: stereo filename source1 source2\n");
    exit (1);
  }

  if ((name = malloc (strlen (argv [1]) + 16)) == NULL) {
    fprintf (stderr, "stereo: malloc failed\n");
    exit (1);
  }

  src1 = atoi (argv [2]);
  src2 = atoi (argv [3]);

  if ((fd = open ("/dev/acvc", O_RDONLY)) < 0) {
    perror ("stereo: open");
    exit (1);
  }
  
  if (ACVC_SETSIZE (fd, WIDTH, HEIGHT, 0)) {
    perror ("stereo: SETSIZE");
    exit (1);
  }
  
  if (ACVC_SETVIEW (fd, WIDTH, HEIGHT)) {
    perror ("stereo: SETVIEW");
    exit (1);
  }
  
  if (ACVC_SETMONO (fd, 0)) {
    perror ("stereo: SETMONO");
    exit (1);
  }
  
  if (ACVC_SETBRI (fd, 192)) {
    perror ("stereo: SETBRI");
    exit (1);
  }
  
  if (ACVC_SETCON (fd, 64)) {
    perror ("stereo: SETCON");
    exit (1);
  }
  
  if (ACVC_SETSAT (fd, 64)) {
    perror ("stereo: SETSAT");
    exit (1);
  }

  /************************************************************/

  c = 0;
  
  if (ACVC_SETSRC (fd, src1)) {
    perror ("stereo: SETSRC");
    exit (1);
  }
  
  if (ACVC_CAPSTART (fd)) {
    perror ("stereo: CAPSTART");
    exit (1);
  }
  
  for (i = 0; i < NUM; i ++) {
    if (ACVC_ISCAPTURING (fd, & n) < 0) {
      perror ("stereo: ISCAPTURING");
      exit (1);
    }

    if (! n) {
      c ++;
    }
    
    if (read (fd, buf, WIDTH * HEIGHT * 2) < 0) {
      perror ("stereo: read");
      exit (1);
    }
    
    if (ACVC_SETSRC (fd, src2)) {
      perror ("stereo: SETSRC");
      exit (1);
    }
    
    if (ACVC_CAPSTART (fd)) {
      perror ("stereo: CAPSTART");
      exit (1);
    }

    if ((j = fork ()) < 0) {
      perror ("stereo: fork");
      exit (1);
    }

    if (! j) {
      sprintf (name, "%s-1-%d.ppm", argv [1], i);

      if ((fp = fopen (name, "w")) == NULL) {
	fprintf (stderr, "stereo: fopen failed\n");
	exit (1);
      }

      fprintf (fp, "P6\n%d %d\n31\n", WIDTH, HEIGHT);
      
      p = (unsigned short *) buf;
      
      for (k = 0; k < WIDTH * HEIGHT; k ++) {
	fputc (ACVC_GETR16 (* p), fp);
	fputc (ACVC_GETG16 (* p), fp);
	fputc (ACVC_GETB16 (* p), fp);

	p ++;
      }
      
      fclose (fp);

      exit (0);
    }

    if (ACVC_ISCAPTURING (fd, & n) < 0) {
      perror ("stereo: ISCAPTURING");
      exit (1);
    }
    
    if (! n) {
      c ++;
    }
    
    if (read (fd, buf, WIDTH * HEIGHT * 2) < 0) {
      perror ("stereo: read");
      exit (1);
    }
    
    if (ACVC_SETSRC (fd, src1)) {
      perror ("stereo: SETSRC");
      exit (1);
    }
    
    if (ACVC_CAPSTART (fd)) {
      perror ("stereo: CAPSTART");
      exit (1);
    }
    
    if ((j = fork ()) < 0) {
      perror ("stereo: fork");
      exit (1);
    }

    if (! j) {
      sprintf (name, "%s-2-%d.ppm", argv [1], i);
      
      if ((fp = fopen (name, "w")) == NULL) {
	fprintf (stderr, "stereo: fopen failed\n");
	exit (1);
      }
      
      fprintf (fp, "P6\n%d %d\n31\n", WIDTH, HEIGHT);
      
      p = (unsigned short *) buf;
      
      for (j = 0; j < WIDTH * HEIGHT; j ++) {
	fputc (ACVC_GETR16 (* p), fp);
	fputc (ACVC_GETG16 (* p), fp);
	fputc (ACVC_GETB16 (* p), fp);

	p ++;
      }
      
      fclose (fp);

      exit (0);
    }
  }

  fprintf (stderr, "%d delays / %d frames\n", c, NUM * 2);

  exit (0);
}
