/*   ArgoCraft Video Capture Board    */
/*   Driver and Sample Applications   */
/* Version 1.32 for Linux and FreeBSD */
/* Copyright (C) 1996 ArgoCraft Corp. */

#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "acvc.h"

#define pad(x) ((x) == 24 ? 32 : (x))
#define fivebits(x) ((x) < 0 ? 0 : (x) >= 32 ? 31 : (x))

static int half, mono, dbl, pal, fd;
static unsigned short x, y;
static unsigned int num, size;
static void * buf;

static Display * disp;
static int scr;
static int depth;
static Visual * vi;
static Colormap cm;
static XColor col;
static XSetWindowAttributes atr;
static Window win;
static XSizeHints hints;
static XImage * img;
static Atom prt, del;
static XEvent ev;

static unsigned char dit [4] [4] [32768];

static const int err [4] [4] = {
  { 2, 0, 2, 0 },
  { 1, 3, 1, 3 },
  { 2, 0, 2, 0 },
  { 1, 3, 1, 3 }
};

static const int erg [4] [4] = {
  { 1, 3, 1, 3 },
  { 2, 0, 2, 0 },
  { 1, 3, 1, 3 },
  { 2, 0, 2, 0 }
};

static const int erb [4] [4] = {
  { 7, 1, 6, 0 },
  { 3, 5, 2, 4 },
  { 6, 0, 7, 1 },
  { 2, 4, 3, 5 }
};

void usage (void) {
  fprintf (stderr, "usage: xvctest [-h] size_x size_y ");
  fprintf (stderr, "view_x view_y mono src bri con sat\n");
  exit (1);
}

int main (int argc, char ** argv) {
  unsigned char dtr [32], dtg [32], dtb [32];
  unsigned long c1 = 0, c2 = 0, t;
  int i, j, k, m, n, r, g, b;
  struct timeval tv1, tv2;
  struct timezone tz;
  unsigned short * ps;
  unsigned long * pl;
  unsigned char * pc, c;

  if (argc <= 1) {
    usage ();
  }

  if (! strcmp (argv [1], "-h")) {
    half = 1;
    argc --;
    argv ++;
  } else {
    half = 0;
  }

  if (argc != 10) {
    usage ();
  }

  mono = atoi (argv [5]);

  /*******************/
  /* X initilization */
  /*******************/

  if (! (disp = XOpenDisplay (""))) {
    fprintf (stderr, "xvctest: can't open display\n");
    exit (1);
  }

  scr = DefaultScreen (disp);
  depth = DefaultDepth (disp, scr);
  vi = DefaultVisual (disp, scr);

  if (depth == 24 && vi -> class == TrueColor) {
    if (vi -> red_mask != 0xff0000 ||
	vi -> green_mask != 0x00ff00 ||
	vi -> blue_mask != 0x0000ff) {
      fprintf (stderr, "xvctest: visual doesn't match\n");
      exit (1);
    }

    dbl = 1;
    pal = 0;
  } else if (depth == 16 && vi -> class == TrueColor) {
    if (vi -> red_mask != 0xf800 ||
	vi -> green_mask != 0x07e0 ||
	vi -> blue_mask != 0x001f) {
      fprintf (stderr, "xvctest: visual doesn't match\n");
      exit (1);
    }

    dbl = mono;
    pal = 0;
  } else {
    dbl = mono;
    pal = 1;
  }

  x = atoi (argv [1]);
  y = atoi (argv [2]);

  num = x * y;

  if (pal) {
    cm = XCreateColormap (disp, RootWindow (disp, scr), vi, AllocAll);

    if (mono) {
      for (i = 0; i < 256; i ++) {
	col . pixel = i;
	col . flags = DoRed | DoGreen | DoBlue;
	col . pad = 0;
	col . red = col . green = col. blue = i << 8;
	
	XStoreColor (disp, cm, & col);
      }
    } else {
      for (i = 0; i < 8; i ++) {
	for (j = 0; j < 8; j ++) {
	  for (k = 0; k < 4; k ++) {
	    col . pixel = i << 5 | j << 2 | k;
	    col . flags = DoRed | DoGreen | DoBlue;
	    col . pad = 0;
	    
	    col . red = i << 13;
	    col . green = j << 13;
	    col . blue = k << 14;
	    
	    XStoreColor (disp, cm, & col);
	  }
	}
      }
    }
  } else {
    cm = DefaultColormap (disp, scr);
  }

  atr . event_mask = NoEventMask;
  atr . colormap = cm;

  win = XCreateWindow (disp, RootWindow (disp, scr), 0, 0, x, y, 1, depth,
		       InputOutput, vi, CWEventMask | CWColormap, & atr);

  hints . flags = PMinSize | PMaxSize;
  hints . min_width = hints . max_width = x;
  hints . min_height = hints . max_height = y;

  XSetWMNormalHints (disp, win, & hints);

  prt = XInternAtom (disp, "WM_PROTOCOLS", False);
  del = XInternAtom (disp, "WM_DELETE_WINDOW", False);

  XSetWMProtocols (disp, win, & del, 1);

  XStoreName (disp, win, "xvctest");
  XSetIconName (disp, win, "xvctest");

  size = num * (dbl ? 4 : 2);

  if (! (buf = malloc (size))) {
    fprintf (stderr, "xvctest: malloc failed\n");
    exit (1);
  }

  img = XCreateImage (disp, vi, depth, ZPixmap, 0, buf, x, y, pad (depth), 0);

  if (! XInitImage (img)) {
    fprintf (stderr, "xvctest: can't initialize image\n");
    exit (1);
  }

  if (img -> byte_order != LSBFirst) {
    fprintf (stderr, "xvctest: byte order doesn't match\n");
    exit (1);
  }

  /**********************/
  /* ACVC initilization */
  /**********************/

  if (half) {
    if ((fd = open ("/dev/acvc0", O_RDONLY)) < 0) {
      perror ("xvctest: open");
      exit (1);
    }
  } else {
    if ((fd = open ("/dev/acvc", O_RDONLY)) < 0) {
      perror ("xvctest: open");
      exit (1);
    }
  }

  if (ACVC_SETSIZE (fd, x, y, dbl)) {
    perror ("xvctest: SETSIZE");
    exit (1);
  }

  if (ACVC_SETVIEW (fd, atoi (argv [3]), atoi (argv [4]))) {
    perror ("xvctest: SETVIEW");
    exit (1);
  }

  if (ACVC_SETMONO (fd, mono)) {
    perror ("xvctest: SETMONO");
    exit (1);
  }

  if (ACVC_SETSRC (fd, atoi (argv [6]))) {
    perror ("xvctest: SETSRC");
    exit (1);
  }

  if (ACVC_SETBRI (fd, atoi (argv [7]))) {
    perror ("xvctest: SETBRI");
    exit (1);
  }

  if (ACVC_SETCON (fd, atoi (argv [8]))) {
    perror ("xvctest: SETCON");
    exit (1);
  }

  if (ACVC_SETSAT (fd, atoi (argv [9]))) {
    perror ("xvctest: SETSAT");
    exit (1);
  }

  /************************/
  /* Dither initilization */
  /************************/

  if (pal && ! mono) {
    for (m = 0; m < 4; m ++) {
      for (n = 0; n < 4; n ++) {
	for (i = 0; i < 32; i ++) {
	  r = i + err [m] [n];
	  g = i + erg [m] [n];
	  b = i + erb [m] [n];
	
	  r = fivebits (r) >> 2;
	  g = fivebits (g) >> 2;
	  b = fivebits (b) >> 3;
	
	  dtr [i] = r << 5;
	  dtg [i] = g << 2;
	  dtb [i] = b;
	}

	for (i = 0; i < 32; i ++) {
	  for (j = 0; j < 32; j ++) {
	    for (k = 0; k < 32; k ++) {
	      dit [m] [n] [i << 10 | j << 5 | k] = dtr [i] | dtg [j] | dtb [k];
	    }
	  }
	}
      }
    }
  }

  /*************/
  /* Main Loop */
  /*************/

  XMapWindow (disp, win);

  if (nice (10)) {
    perror ("xvctest: nice");
    exit (1);
  }

  if (gettimeofday (& tv1, & tz)) {
    perror ("xvctest: gettimeofday");
    exit (1);
  }

  if (ACVC_CAPSTART (fd)) {
    perror ("xvctest: CAPSTART");
    exit (1);
  }

  while (1) {
    while (XPending (disp)) {
      XNextEvent (disp, & ev);

      if (ev . type == ClientMessage &&
	  ev . xclient . message_type == prt &&
	  ev . xclient . data . l [0] == del) {
	if (gettimeofday (& tv2, & tz)) {
	  perror ("xvctest: gettimeofday");
	  exit (1);
	}

	t = (tv2 . tv_usec - tv1 . tv_usec)
	  + (tv2 . tv_sec - tv1 . tv_sec) * 1000000;

	printf ("%.3f secs, ", (double) t / 1000000);
	printf ("%ld delays / %ld frames, ", c2 - c1, c2);
	printf ("%.3f fps\n", (double) c2 * 1000000 / t);

	exit (0);
      }
    }

    if (ACVC_ISCAPTURING (fd, & n)) {
      perror ("xvctest: ISCAPTURING");
      exit (1);
    }

    if (n) {
      c1 ++;
    }
    c2 ++;

    if ((n = read (fd, buf, size)) < 0) {
      perror ("xvctest: read");
      exit (1);
    }
    
    if (ACVC_CAPSTART (fd)) {
      perror ("xvctest: CAPSTART");
      exit (1);
    }
    
    if (n != size) {
      fprintf (stderr, "xvctest: read failed\n");
      exit (1);
    }

    switch (depth) {
    case 8:
      if (mono) {
	pl = buf;
	pc = buf;

	for (i = num; i > 0; i --) {
	  * pc ++ = * (unsigned char *) pl ++;
	}
      } else {
	ps = buf;
	pc = buf;
	
	for (j = 0; j < y; j ++) {
	  for (i = 0; i < x; i ++) {
	    * pc ++ = dit [i & 3] [j & 3] [* ps ++];
	  }
	}
      }

      break;
    case 16:
      if (mono) {
	pl = buf;
	ps = buf;

	for (i = num; i > 0; i --) {
	  c = * (unsigned char *) pl ++;
	  * ps ++ = (c >> 3 << 11) | (c >> 2 << 5) | (c >> 3);
	}
      } else {
	ps = buf;
	
	for (i = num; i > 0; i --) {
	  * ps = (* ps & 0x001f) | (* ps & 0xffe0) << 1;
	  ps ++;
	}
      }

      break;
    case 24:
      if (mono) {
	;
      } else {
	;
      }

      break;
    default:
      ;
    }

    XPutImage (disp, win, DefaultGC (disp, scr), img, 0, 0, 0, 0, x, y);
  }
}
