/*   ArgoCraft Video Capture Board    */
/*   Driver and Sample Applications   */
/* Version 1.32 for Linux and FreeBSD */
/* Copyright (C) 1996 ArgoCraft Corp. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <vga.h>
#include <vgagl.h>
#include "acvc.h"

void usage (void) {
  fprintf (stderr, "usage: svctest [-c] [-h] source\n");
  exit (1);
}

int main (int argc, char ** argv) {
  unsigned long * lp, * lp1, * lp2, * lp1i, * lp2i;
  unsigned short * sp, * sp1, * sp2, * sp1i, * sp2i;
  int fd, color, half, size, x, y, i, j;
  void * buf;

  if (vga_init ()) {
    exit (1);
  }

  if (argc < 2) {
    usage ();
  }

  if (! strcmp (argv [1], "-c")) {
    argc --;
    argv ++;

    color = 1;
  } else {
    color = 0;
  }

  if (argc < 2) {
    usage ();
  }

  if (! strcmp (argv [1], "-h")) {
    argc --;
    argv ++;

    half = 1;

    x = 320;
    y = 240;
  } else {
    half = 0;

    x = 640;
    y = 480;
  }

  if (argc != 2) {
    usage ();
  }

  if ((buf = malloc (640 * 480 * (color ? 4 : 2))) == NULL) {
    fprintf (stderr, "svctest: malloc failed\n");
    exit (1);
  }

  size = x * y * (color ? 4 : 2);

  lp1i = buf;
  lp1i += 320 * 240;
  
  lp2i = buf;
  lp2i += 640 * 480;
  
  sp1i = buf;
  sp1i += 320 * 240;
  
  sp2i = buf;
  sp2i += 640 * 480;

  if (! vga_hasmode (color ? G640x480x16M32 : G640x480x32K)) {
    fprintf (stderr,
	     "svctest: mode %s not available\n",
	     color ? "G640x480x16M32" : "G640x480x32K");
    exit (1);
  }

  vga_setmode (color ? G640x480x16M32 : G640x480x32K);

  gl_setcontextvga (color ? G640x480x16M32 : G640x480x32K);
  gl_clearscreen (0);

  if ((fd = open (half ? "/dev/acvc0" : "/dev/acvc", O_RDONLY)) < 0) {
    perror ("svctest: open");
    exit (1);
  }

  if (ACVC_SETSIZE (fd, x, y, color)) {
    perror ("svctest: SETSIZE");
    exit (1);
  }

  if (ACVC_SETVIEW (fd, 640, 480)) {
    perror ("svctest: SETVIEW");
    exit (1);
  }

  if (ACVC_SETMONO (fd, 0)) {
    perror ("svctest: SETMONO");
    exit (1);
  }

  if (ACVC_SETSRC (fd, atoi (argv [1]))) {
    perror ("svctest: SETSRC");
    exit (1);
  }

  if (ACVC_SETBRI (fd, 192)) {
    perror ("svctest: SETBRI");
    exit (1);
  }

  if (ACVC_SETCON (fd, 64)) {
    perror ("svctest: SETCON");
    exit (1);
  }

  if (ACVC_SETSAT (fd, 64)) {
    perror ("svctest: SETSAT");
    exit (1);
  }

  if (ACVC_CAPSTART (fd)) {
    perror ("svctest: CAPSTART");
    exit (1);
  }

  while (1) {
    if (read (fd, buf, size) < 0) {
      perror ("svctest: read");
      exit (1);
    }

    if (ACVC_CAPSTART (fd)) {
      perror ("svctest: CAPSTART");
      exit (1);
    }

    if (half) {
      if (color) {
	lp1 = lp1i;
	lp2 = lp2i;

	for (i = 240; i > 0; i --) {
	  lp = lp1;

	  for (j = 320; j > 0; j --) {
	    lp1 --;

	    lp2 --;
	    * lp2 = * lp1;

	    lp2 --;
	    * lp2 = * lp1;
	  }

	  lp1 = lp;

	  for (j = 320; j > 0; j --) {
	    lp1 --;

	    lp2 --;
	    * lp2 = * lp1;

	    lp2 --;
	    * lp2 = * lp1;
	  }
	}
      } else {
	sp1 = sp1i;
	sp2 = sp2i;

	for (i = 240; i > 0; i --) {
	  sp = sp1;

	  for (j = 320; j > 0; j --) {
	    sp1 --;

	    sp2 --;
	    * sp2 = * sp1;

	    sp2 --;
	    * sp2 = * sp1;
	  }

	  sp1 = sp;

	  for (j = 320; j > 0; j --) {
	    sp1 --;

	    sp2 --;
	    * sp2 = * sp1;

	    sp2 --;
	    * sp2 = * sp1;
	  }
	}
      }
    }

    gl_putbox (0, 0, 640, 480, buf);
  }

  vga_setmode (TEXT);

  return 0;
}
