/*   ArgoCraft Video Capture Board    */
/*   Driver and Sample Applications   */
/* Version 1.32 for Linux and FreeBSD */
/* Copyright (C) 1996 ArgoCraft Corp. */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <vga.h>
#include <vgagl.h>
#include "acvc.h"

void usage (void) {
  fprintf (stderr, "usage: svctest [-c] source\n");
  exit (1);
}

int main (int argc, char ** argv) {
  int fd, color, turn, line;
  unsigned short * sp;
  unsigned long * lp;
  void * buf;

  if (vga_init ()) {
    exit (1);
  }

  if (argc < 2) {
    usage ();
  }

  if (! strcmp (argv [1], "-c")) {
    argc --;
    argv ++;

    color = 1;
  } else {
    color = 0;
  }

  if (argc != 2) {
    usage ();
  }

  if ((buf = malloc (640 * 240 * (color ? 4 : 2))) == NULL) {
    fprintf (stderr, "svctest: malloc failed\n");
    exit (1);
  }

  if (! vga_hasmode (color ? G640x480x16M32 : G640x480x32K)) {
    fprintf (stderr,
	     "svctest: mode %s not available\n",
	     color ? "G640x480x16M32" : "G640x480x32K");
    exit (1);
  }

  vga_setmode (color ? G640x480x16M32 : G640x480x32K);

  gl_setcontextvga (color ? G640x480x16M32 : G640x480x32K);
  gl_clearscreen (0);

  if ((fd = open ("/dev/acvci", O_RDONLY)) < 0) {
    perror ("svctest: open");
    exit (1);
  }

  if (ACVC_SETSIZE (fd, 640, 240, color)) {
    perror ("svctest: SETSIZE");
    exit (1);
  }

  if (ACVC_SETVIEW (fd, 640, 480)) {
    perror ("svctest: SETVIEW");
    exit (1);
  }

  if (ACVC_SETMONO (fd, 0)) {
    perror ("svctest: SETMONO");
    exit (1);
  }

  if (ACVC_SETSRC (fd, atoi (argv [1]))) {
    perror ("svctest: SETSRC");
    exit (1);
  }

  if (ACVC_SETBRI (fd, 192)) {
    perror ("svctest: SETBRI");
    exit (1);
  }

  if (ACVC_SETCON (fd, 64)) {
    perror ("svctest: SETCON");
    exit (1);
  }

  if (ACVC_SETSAT (fd, 64)) {
    perror ("svctest: SETSAT");
    exit (1);
  }

  if (ACVC_CAPSTART (fd)) {
    perror ("svctest: CAPSTART");
    exit (1);
  }

  turn = 0;

  while (1) {
    if (read (fd, buf, 640 * 240 * (color ? 4 : 2)) < 0) {
      perror ("svctest: read");
      exit (1);
    }

    if (ACVC_CAPSTART (fd)) {
      perror ("svctest: CAPSTART");
      exit (1);
    }

    sp = buf;
    lp = buf;
    line = 0;
    
    if (turn) {
      sp += 640;
      lp += 640;
      line ++;
    }
    
    while (line < 480) {
      gl_putbox (0, line, 640, 1, color ? (void *) lp : (void *) sp);

      sp += 640;
      lp += 640;
      line += 2;
    }

    turn = ! turn;
  }

  vga_setmode (TEXT);

  return 0;
}
